﻿/*********************************************************************
 * The MIT License (MIT)
 * 
 * Copyright (c) 2010 Manish Sinha <mail@manishsinha.net>
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:

 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 ********************************************************************/

namespace LpNet.WadlSharp.Common.Generator
{
    using System.CodeDom;
    using System.Collections.Generic;
    using Enum;
    using Interface.Generator;
    using Resources;

    public class LpClass : IClass
    {
        /// <summary>
        /// Create a CodeDeclarationType (Class)
        /// </summary>
        /// <param name="repType">The representation instance which has non-null params Property</param>
        /// <returns>Instance of CodeTypeDeclaration</returns>
        public CodeTypeDeclaration Create(representation_type repType)
        {
            CodeTypeDeclaration cdDec= new CodeTypeDeclaration();

            cdDec.Name = repType.id;
            cdDec.IsClass = true;
            cdDec.Attributes = MemberAttributes.Public | MemberAttributes.Static;

            #region Creating the XML Comments for this class

            IComment comment = new LpComment();
            if (repType.doc != null)
                foreach (doc doc in repType.doc)
                {
                    cdDec.Comments.AddRange(comment.Create(doc, LanguageConstructType.Class,repType.id));
                }

            #endregion

            #region Creating the Properties

            if (repType.param != null)
            {
                List<CodeMemberProperty> propertiesList= new List<CodeMemberProperty>();
                List<CodeMemberField> fieldsList = new List<CodeMemberField>();

                // Iterate over all the Properties
                foreach (param param in repType.param)
                {
                    CodeMemberField field;

                    LpProperty property = new LpProperty();
                    CodeMemberProperty prop = property.Create(param, out field);

                    propertiesList.Add(prop);
                    fieldsList.Add(field);
                }

                // Add the Fields and Properties for this Type
                cdDec.Members.AddRange(fieldsList.ToArray());
                cdDec.Members.AddRange(propertiesList.ToArray());
            }

            #endregion

            return cdDec;
        }
    }
}
